import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * Class implementing a TreeSet with the notion of IOrderable
 * 
 * @author Gloss
 * @version 1.1 17/Jul/2013 as
 * 
 * @param <K> Type of the set
 */
public class TreeSet <K extends IOrderable<K>> implements Cloneable, Iterable<K> {

	/**
	 * The tree
	 */
	private List<K> tree;

	/**
	 * Constructor
	 */
	public TreeSet() {
		tree = new ArrayList<K>();
	}

	/**
	 * Inserts an element into the TreeSet
	 * 
	 * @param elem - element to insert
	 */
	public void insert(K elem) {
		if (!isIn(elem))
			tree.add(elem);
	}

	/**
	 * Inserts all the elements of a Set into this TreeSet
	 * 
	 * @param set - the set with the elements to insert
	 */
	public void insertAll(Set<K> set) {    
		for (K elem : set)
			insert(elem); 
	}

	/**
	 * Is this TreeSet empty?
	 * 
	 * @return true if the TreeSet has no elements, false otherwise
	 */
	public boolean isEmpty() {
		return size()==0;
	}

	/**
	 * The size of the TreeSet
	 * 
	 * @return the number of elements in the TreeSet
	 */
	public int size() {
		return tree.size();
	}

	/**
	 * Is the given element in this TreeSet?
	 * 
	 * @param elem - element to search
	 * @return true if the element is in the set, false otherwise
	 */
	public boolean isIn(K elem) {
		return tree.contains(elem);
	}

	/**
	 * Returns the largest element of this TreeSet
	 * 
	 * @return largest element of the TreeSet
	 */
	public K largest() {
		K max = tree.get(0);
		for (int i = 1; i < tree.size(); i++) {
			K aux = tree.get(i);
			if (aux.isGreaterEq(max))
				max = aux;
		}
		return max;
	}

	/**
	 * The iterator for this TreeSet
	 * 
	 * @return iterator for this TreeSet
	 */
	public Iterator<K> iterator(){
		return tree.iterator();
	}

	/**
	 * Makes a copy of this TreeSet
	 * 
	 * @return clone of this TreeSet
	 */
	@SuppressWarnings("unchecked")
	public TreeSet<K> clone() {
		TreeSet<K> result = null;		
		try {
			result = (TreeSet<K>) super.clone();
			result.tree = new ArrayList<K>();
			for (K elem : tree) {
				result.tree.add((K) elem.clone());
			}
		} catch (CloneNotSupportedException e) {
			throw new InternalError(e.getMessage());
		}
		return result;
	}

	/**
	 * Returns the hash code for this TreeSet
	 * 
	 * @return hash code for this TreeSet
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((tree == null) ? 0 : tree.size());
		for (K elem : tree) {
			result = prime * result + elem.hashCode();
		}
		return result;
	}

	/**
	 * Are the two objects equal?
	 * 
	 * @param obj - the object to compare
	 * @return true if the object is equal to this TreeSet, false otherwise
	 */
	@SuppressWarnings("unchecked")
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		return (equalsTrees((TreeSet<K>) obj));
	}	

	/**
	 * Are the two TreeSets equal? 
	 * @param other - TreeSet to be compared to this
	 * @return true if they are equal, false otherwise
	 */
	public boolean equalsTrees(TreeSet<K> other){
		if (tree == null)
			return (other.tree == null);
		if(other.tree.size() != this.tree.size())
			return false;
		for (K elem : this.tree) {
			if (!other.tree.contains(elem)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Textual representation of this TreeSet
	 * 
	 * @return String representing the TreeSet
	 */
	public String toString() {

		StringBuilder result = new StringBuilder(); 
		result.append("TreeSet {");
		for (K e: tree) result.append(e + ",");
		result.deleteCharAt(result.length()-1);
		result.append("}");
		return result.toString();
	}
}